@@ -1,5 +1,6 @@
 import httplib
 import urllib
+import xml
 from datetime import datetime, timedelta
 from xml.dom import minidom
 from django.db import models
@@ -11,7 +12,7 @@ class CachedDocumentManager(models.Manager):
     """
     This manager handles querying or retrieving CachedDocuments.
     """
-    def cache_from_eve_api(self, cached_doc, url_path, params):
+    def cache_from_eve_api(self, cached_doc, url_path, params, no_cache=False):
         """
         Connect to the EVE API server, send the request, and cache it to
         a CachedDocument. This is typically not something you want to call
@@ -25,22 +26,30 @@ class CachedDocumentManager(models.Manager):
         # Retrieve the response from the server.
         response = conn.getresponse()
         # Save the response (an XML document) to the CachedDocument.
-        cached_doc.body = unicode(response.read(), 'utf-8').encode('utf-8')
+        cached_doc.body = response.read()
     
-        # Parse the response via minidom
-        dom = minidom.parseString(cached_doc.body)
+        try:
+            # Parse the response via minidom
+            dom = minidom.parseString(cached_doc.body)
+        except xml.parsers.expat.ExpatError:
+            print "XML Parser Error:"
+            print cached_doc.body
+            return
+
         # Set the CachedDocument's time_retrieved and cached_until times based
         # on the values in the XML response. This will be used in future
         # requests to see if the CachedDocument can be retrieved directly or
         # if it needs to be re-cached.
-        cached_doc.time_retrieved = dom.getElementsByTagName('currentTime')[0].childNodes[0].nodeValue
+        cached_doc.time_retrieved = datetime.utcnow()
         cached_doc.cached_until = dom.getElementsByTagName('cachedUntil')[0].childNodes[0].nodeValue
     
         # Finish up and return the resulting document just in case.
-        cached_doc.save()
+        if no_cache == False:
+            cached_doc.save()
+
         return cached_doc
     
-    def api_query(self, url_path, params=None):
+    def api_query(self, url_path, params=None, no_cache=False):
         """
         Transparently handles querying EVE API or retrieving the document from
         the cache.
@@ -64,20 +73,26 @@ class CachedDocumentManager(models.Manager):
         # Combine the URL path and the parameters to create the full query.
         query_name = '%s?%s' % (url_path, params)
         
-        # Retrieve or create a new CachedDocument based on the full URL
-        # and parameters.
-        cached_doc, created = self.get_or_create(url_path=query_name)
+        if no_cache:
+            # If no_cache is enabled, don't even attempt a lookup.
+            cached_doc = CachedDocument()
+            created = False
+        else:
+            # Retrieve or create a new CachedDocument based on the full URL
+            # and parameters.
+            cached_doc, created = self.get_or_create(url_path=query_name)
     
         # EVE uses UTC.
         current_eve_time = datetime.utcnow()
 
         # Figure out if we need hit EVE API and re-cache, or just pull from
         # the local cache (based on cached_until).
-        if created or \
+        if no_cache or created or \
           cached_doc.cached_until == None or \
           current_eve_time > cached_doc.cached_until:
             # Cache from EVE API
-            self.cache_from_eve_api(cached_doc, url_path, params)
+            self.cache_from_eve_api(cached_doc, url_path, params, 
+                                    no_cache=no_cache)
             
         return cached_doc
 
